@@ -87,16 +87,11 @@ print_boulder(int io_version,
     
   /* This deals with the renaming of the internal oligo */
   const char *new_oligo_name = "INTERNAL";
-  const char *old_oligo_name = "INTERNAL_OLIGO";
   char *int_oligo = (char*) new_oligo_name;
 
   /* Check: are all pointers linked to something*/
   PR_ASSERT(NULL != pa);
   PR_ASSERT(NULL != sa);
-    
-  if (io_version == 3) {
-    int_oligo = (char*) old_oligo_name;
-  }
         
   /* Check if there are warnings and print them */
   if ((warning = p3_get_rv_and_gs_warnings(retval, pa)) != NULL) { 
@@ -236,21 +231,12 @@ print_boulder(int io_version,
     }
       
     /* Get the number for pimer counting in suffix[0] */
-    if ((i == 0) && (io_version == 3) ){
-      suffix[0] = '\0';
-    } else { 
-      sprintf(suffix, "_%d", i);
-    }
-
+    sprintf(suffix, "_%d", i);
+    
     /* Print out the Pair Penalties */
     if (retval->output_type == primer_pairs) {
-      if (io_version == 3) {
-        printf("PRIMER_PAIR_PENALTY%s=%.4f\n", suffix,
-               retval->best_pairs.pairs[i].pair_quality);
-      } else {
-        printf("PRIMER_PAIR%s_PENALTY=%f\n", suffix,
-               retval->best_pairs.pairs[i].pair_quality);
-      }
+      printf("PRIMER_PAIR%s_PENALTY=%f\n", suffix,
+	     retval->best_pairs.pairs[i].pair_quality);
     }
 
     /* Print single primer penalty */
@@ -314,99 +300,76 @@ print_boulder(int io_version,
              intl->gc_content);
 
     /* Print primer self_any */
-    if (go_fwd == 1 && pa->thermodynamic_alignment==0)
+    if (go_fwd == 1 && pa->thermodynamic_oligo_alignment==0)
       printf("PRIMER_LEFT%s_SELF_ANY=%.2f\n", suffix,
              fwd->self_any);
-    if (go_rev == 1 && pa->thermodynamic_alignment==0)
+    if (go_rev == 1 && pa->thermodynamic_oligo_alignment==0)
       printf("PRIMER_RIGHT%s_SELF_ANY=%.2f\n", suffix,
              rev->self_any);
-    if (go_int == 1 && pa->thermodynamic_alignment==0)
+    if (go_int == 1 && pa->thermodynamic_oligo_alignment==0)
       printf("PRIMER_%s%s_SELF_ANY=%.2f\n", int_oligo, suffix,
              intl->self_any);
-     if (go_int == 1 && pa->thermodynamic_alignment==1)
+     if (go_int == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_%s%s_SELF_ANY_TH=%.2f\n", int_oligo, suffix,
 	      intl->self_any);
     /* Print primer self_any thermodynamical approach */
-     if (go_fwd == 1 && pa->thermodynamic_alignment==1)
+     if (go_fwd == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_LEFT%s_SELF_ANY_TH=%.2f\n", suffix,
 	      fwd->self_any);
-     if (go_rev == 1 && pa->thermodynamic_alignment==1)
+     if (go_rev == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_RIGHT%s_SELF_ANY_TH=%.2f\n", suffix,
 	      rev->self_any);
     /* Print primer self_end*/
-    if (go_fwd == 1 && pa->thermodynamic_alignment==0)
+    if (go_fwd == 1 && pa->thermodynamic_oligo_alignment==0)
       printf("PRIMER_LEFT%s_SELF_END=%.2f\n", suffix,
              fwd->self_end);
-    if (go_rev == 1 && pa->thermodynamic_alignment==0)
+    if (go_rev == 1 && pa->thermodynamic_oligo_alignment==0)
       printf("PRIMER_RIGHT%s_SELF_END=%.2f\n", suffix,
              rev->self_end);
-    if (go_int == 1 && pa->thermodynamic_alignment==0)
+    if (go_int == 1 && pa->thermodynamic_oligo_alignment==0)
       printf("PRIMER_%s%s_SELF_END=%.2f\n", int_oligo, suffix,
              intl->self_end);
-     if (go_int == 1 && pa->thermodynamic_alignment==1)
+     if (go_int == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_%s%s_SELF_END_TH=%.2f\n", int_oligo, suffix,
 	      intl->self_end);
      /* Print primer self_end thermodynamical approach */
-     if (go_fwd == 1 && pa->thermodynamic_alignment==1)
+     if (go_fwd == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_LEFT%s_SELF_END_TH=%.2f\n", suffix,
 	      fwd->self_end);
-     if (go_rev == 1 && pa->thermodynamic_alignment==1)
+     if (go_rev == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_RIGHT%s_SELF_END_TH=%.2f\n", suffix,
 	      rev->self_end);
      /* Print primer hairpin */
-     if (go_fwd == 1 && pa->thermodynamic_alignment==1)
+     if (go_fwd == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_LEFT%s_HAIRPIN_TH=%.2f\n", suffix,
 	      fwd->hairpin_th);
-     if (go_rev == 1 && pa->thermodynamic_alignment==1)
+     if (go_rev == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_RIGHT%s_HAIRPIN_TH=%.2f\n", suffix,
 	      rev->hairpin_th);
-     if (go_int == 1 && pa->thermodynamic_alignment==1)
+     if (go_int == 1 && pa->thermodynamic_oligo_alignment==1)
        printf("PRIMER_%s%s_HAIRPIN_TH=%.2f\n", int_oligo, suffix,
 	      intl->hairpin_th);
      /*Print out primer mispriming scores */
-    if (io_version == 3) {
-		if (seq_lib_num_seq(pa->p_args.repeat_lib) > 0) {
-		  if (go_fwd == 1)
-		    printf("PRIMER_LEFT%s_MISPRIMING_SCORE=%.2f, %s\n", suffix,
-		           fwd->repeat_sim.score[fwd->repeat_sim.max],
-		           fwd->repeat_sim.name);
-		  if (go_rev == 1)
-		    printf("PRIMER_RIGHT%s_MISPRIMING_SCORE=%.2f, %s\n", suffix,
-		           rev->repeat_sim.score[rev->repeat_sim.max],
-		           rev->repeat_sim.name);
-		  if (retval->output_type == primer_pairs)
-		    printf("PRIMER_PAIR%s_MISPRIMING_SCORE=%.2f, %s\n", suffix,
-		           retval->best_pairs.pairs[i].repeat_sim,
-		           retval->best_pairs.pairs[i].rep_name);
-		}
-    
-		/* Print out internal oligo mispriming scores */
-		if (go_int == 1 && seq_lib_num_seq(pa->o_args.repeat_lib) > 0)
-		  printf("PRIMER_%s%s_MISHYB_SCORE=%.2f, %s\n", int_oligo, suffix,
-		         intl->repeat_sim.score[intl->repeat_sim.max],
-		         intl->repeat_sim.name);
-    } else {
-		if (seq_lib_num_seq(pa->p_args.repeat_lib) > 0) {
-		  if (go_fwd == 1)
-		    printf("PRIMER_LEFT%s_LIBRARY_MISPRIMING=%.2f, %s\n", suffix,
-		           fwd->repeat_sim.score[fwd->repeat_sim.max],
-		           fwd->repeat_sim.name);
-		  if (go_rev == 1)
-		    printf("PRIMER_RIGHT%s_LIBRARY_MISPRIMING=%.2f, %s\n", suffix,
-		           rev->repeat_sim.score[rev->repeat_sim.max],
-		           rev->repeat_sim.name);
-		  if (retval->output_type == primer_pairs)
-		    printf("PRIMER_PAIR%s_LIBRARY_MISPRIMING=%.2f, %s\n", suffix,
-		           retval->best_pairs.pairs[i].repeat_sim,
-		           retval->best_pairs.pairs[i].rep_name);
-		}
+     if (seq_lib_num_seq(pa->p_args.repeat_lib) > 0) {
+       if (go_fwd == 1)
+	 printf("PRIMER_LEFT%s_LIBRARY_MISPRIMING=%.2f, %s\n", suffix,
+		fwd->repeat_sim.score[fwd->repeat_sim.max],
+		fwd->repeat_sim.name);
+       if (go_rev == 1)
+	 printf("PRIMER_RIGHT%s_LIBRARY_MISPRIMING=%.2f, %s\n", suffix,
+		rev->repeat_sim.score[rev->repeat_sim.max],
+		rev->repeat_sim.name);
+       if (retval->output_type == primer_pairs)
+	 printf("PRIMER_PAIR%s_LIBRARY_MISPRIMING=%.2f, %s\n", suffix,
+		retval->best_pairs.pairs[i].repeat_sim,
+		retval->best_pairs.pairs[i].rep_name);
+     }
 		
-		/* Print out internal oligo mispriming scores */
-		if (go_int == 1 && seq_lib_num_seq(pa->o_args.repeat_lib) > 0)
-		  printf("PRIMER_%s%s_LIBRARY_MISHYB=%.2f, %s\n", int_oligo, suffix,
-		         intl->repeat_sim.score[intl->repeat_sim.max],
-		         intl->repeat_sim.name);
-    }
+     /* Print out internal oligo mispriming scores */
+     if (go_int == 1 && seq_lib_num_seq(pa->o_args.repeat_lib) > 0)
+       printf("PRIMER_%s%s_LIBRARY_MISHYB=%.2f, %s\n", int_oligo, suffix,
+	      intl->repeat_sim.score[intl->repeat_sim.max],
+	      intl->repeat_sim.name);
 
     /* If a sequence quality was provided, print it*/
     if (NULL != sa->quality){
@@ -440,23 +403,33 @@ print_boulder(int io_version,
              suffix, rev->end_stability);
 
     /* Print primer template mispriming */
-    if ( (pa->thermodynamic_alignment == 0) && (go_fwd == 1) && 
+    if ( (pa->thermodynamic_template_alignment == 0) && (go_fwd == 1) && 
          (oligo_max_template_mispriming(fwd) != ALIGN_SCORE_UNDEF))
       printf("PRIMER_LEFT%s_TEMPLATE_MISPRIMING=%.4f\n", suffix,
              oligo_max_template_mispriming(fwd));
-    if ( (pa->thermodynamic_alignment == 0) && (go_rev == 1) && 
+    if ( (pa->thermodynamic_template_alignment == 0) && (go_rev == 1) && 
          (oligo_max_template_mispriming(rev) != ALIGN_SCORE_UNDEF))
       printf("PRIMER_RIGHT%s_TEMPLATE_MISPRIMING=%.4f\n", suffix,
              oligo_max_template_mispriming(rev));
-     /* Print primer template mispriming thermodynamical approach*/
-     if ( (pa->thermodynamic_alignment == 1) && (go_fwd == 1) &&
-	  (oligo_max_template_mispriming_thermod(fwd) != ALIGN_SCORE_UNDEF))
+
+     /* Print primer template mispriming, thermodynamical approach*/
+     if ( (pa->thermodynamic_template_alignment == 1) && (go_fwd == 1) &&
+	  (oligo_max_template_mispriming_thermod(fwd) != ALIGN_SCORE_UNDEF)) {
        printf("PRIMER_LEFT%s_TEMPLATE_MISPRIMING_TH=%.4f\n", suffix,
 	      oligo_max_template_mispriming_thermod(fwd));
-     if ( (pa->thermodynamic_alignment == 1) && (go_rev == 1) &&
-	  (oligo_max_template_mispriming_thermod(rev) != ALIGN_SCORE_UNDEF))
+     }
+
+     if ( (pa->thermodynamic_template_alignment == 1) && (go_rev == 1) &&
+	  (oligo_max_template_mispriming_thermod(rev) != ALIGN_SCORE_UNDEF)) {
        printf("PRIMER_RIGHT%s_TEMPLATE_MISPRIMING_TH=%.4f\n", suffix,
 	      oligo_max_template_mispriming_thermod(rev));
+#if 0
+       printf("DEBUG_PRIMER_RIGHT%s_TEMPLATE_MISPRIMING_TOP_TH=%.4f\n", suffix,
+	      rev->template_mispriming);
+       printf("DEBUG_PRIMER_RIGHT%s_TEMPLATE_MISPRIMING_R_TH=%.4f\n", suffix,
+	      rev->template_mispriming_r);
+#endif
+     }
      /************************************************************************************/
      /* Print the pair parameters*/
     if (retval->output_type == primer_pairs) {
@@ -464,59 +437,41 @@ print_boulder(int io_version,
         printf("PRIMER_%s%s_MIN_SEQ_QUALITY=%d\n", int_oligo,
                suffix, intl->seq_quality);
       /* Print pair comp_any */
-       if(pa->thermodynamic_alignment==0)
+       if(pa->thermodynamic_oligo_alignment==0)
 	 printf("PRIMER_PAIR%s_COMPL_ANY=%.2f\n", suffix,
 		retval->best_pairs.pairs[i].compl_any);
-       if(pa->thermodynamic_alignment==1)
+       if(pa->thermodynamic_oligo_alignment==1)
 	 printf("PRIMER_PAIR%s_COMPL_ANY_TH=%.2f\n", suffix,
 		retval->best_pairs.pairs[i].compl_any);
        /* Print pair comp_end */
-       if(pa->thermodynamic_alignment==0)
+       if(pa->thermodynamic_oligo_alignment==0)
 	 printf("PRIMER_PAIR%s_COMPL_END=%.2f\n", suffix,
 		retval->best_pairs.pairs[i].compl_end);
-       if(pa->thermodynamic_alignment==1)
+       if(pa->thermodynamic_oligo_alignment==1)
 	 printf("PRIMER_PAIR%s_COMPL_END_TH=%.2f\n", suffix,
 		retval->best_pairs.pairs[i].compl_end);
-       if (io_version == 3) {
-	  /* Print product size */
-	  printf("PRIMER_PRODUCT_SIZE%s=%d\n", suffix,
-		 retval->best_pairs.pairs[i].product_size);
-        /* Print the product Tm if a Tm range is defined */
-        if (pa->product_max_tm != PR_DEFAULT_PRODUCT_MAX_TM ||
-            pa->product_min_tm != PR_DEFAULT_PRODUCT_MIN_TM) {
-          printf("PRIMER_PRODUCT_TM%s=%.4f\n", suffix,
-                 retval->best_pairs.pairs[i].product_tm);
-                    
-          printf("PRIMER_PRODUCT_TM_OLIGO_TM_DIFF%s=%.4f\n", suffix,
-                 retval->best_pairs.pairs[i].product_tm_oligo_tm_diff);
-                
-          printf("PRIMER_PAIR%s_T_OPT_A=%.4f\n", suffix,
-                 retval->best_pairs.pairs[i].t_opt_a);
-        }
-      } else {
-        /* Print product size */
-        printf("PRIMER_PAIR%s_PRODUCT_SIZE=%d\n", suffix,
-               retval->best_pairs.pairs[i].product_size);
-        /* Print the product Tm if a Tm range is defined */
-        if (pa->product_max_tm != PR_DEFAULT_PRODUCT_MAX_TM ||
-            pa->product_min_tm != PR_DEFAULT_PRODUCT_MIN_TM) {
-          printf("PRIMER_PAIR%s_PRODUCT_TM=%.4f\n", suffix,
-                 retval->best_pairs.pairs[i].product_tm);
-                    
-          printf("PRIMER_PAIR%s_PRODUCT_TM_OLIGO_TM_DIFF=%.4f\n", suffix,
-                 retval->best_pairs.pairs[i].product_tm_oligo_tm_diff);
-                
-          printf("PRIMER_PAIR%s_T_OPT_A=%.4f\n", suffix,
-                 retval->best_pairs.pairs[i].t_opt_a);
-        }
-      }
+       /* Print product size */
+       printf("PRIMER_PAIR%s_PRODUCT_SIZE=%d\n", suffix,
+	      retval->best_pairs.pairs[i].product_size);
+       /* Print the product Tm if a Tm range is defined */
+       if (pa->product_max_tm != PR_DEFAULT_PRODUCT_MAX_TM ||
+	   pa->product_min_tm != PR_DEFAULT_PRODUCT_MIN_TM) {
+	 printf("PRIMER_PAIR%s_PRODUCT_TM=%.4f\n", suffix,
+		retval->best_pairs.pairs[i].product_tm);
+	 
+	 printf("PRIMER_PAIR%s_PRODUCT_TM_OLIGO_TM_DIFF=%.4f\n", suffix,
+		retval->best_pairs.pairs[i].product_tm_oligo_tm_diff);
+	 
+	 printf("PRIMER_PAIR%s_T_OPT_A=%.4f\n", suffix,
+		retval->best_pairs.pairs[i].t_opt_a);
+       }
       
-      /* Print the primer pair temlate mispriming */
-      if ((pa->thermodynamic_alignment == 0) && (retval->best_pairs.pairs[i].template_mispriming != ALIGN_SCORE_UNDEF))
+      /* Print the primer pair template mispriming */
+      if ((pa->thermodynamic_template_alignment == 0) && (retval->best_pairs.pairs[i].template_mispriming != ALIGN_SCORE_UNDEF))
         printf("PRIMER_PAIR%s_TEMPLATE_MISPRIMING=%.2f\n", suffix,
                retval->best_pairs.pairs[i].template_mispriming);
-       /* Print the primer pair temlate mispriming. Thermodynamic approach.  */
-       if ((pa->thermodynamic_alignment == 1) && (retval->best_pairs.pairs[i].template_mispriming != ALIGN_SCORE_UNDEF))
+       /* Print the primer pair template mispriming. Thermodynamic approach.  */
+       if ((pa->thermodynamic_template_alignment == 1) && (retval->best_pairs.pairs[i].template_mispriming != ALIGN_SCORE_UNDEF))
 	 printf("PRIMER_PAIR%s_TEMPLATE_MISPRIMING_TH=%.2f\n", suffix,
 		retval->best_pairs.pairs[i].template_mispriming);
 
@@ -570,8 +525,7 @@ print_all_explain(const p3_global_settings *pa,
 
   if ( pa->pick_internal_oligo == 1
       && !(pa->pick_anyway && sa->internal_input)) 
-    printf("PRIMER_INTERNAL_%sEXPLAIN=%s\n",
-           (io_version == 3 ? "OLIGO_" : ""),
+    printf("PRIMER_INTERNAL_EXPLAIN=%s\n",
            p3_get_oligo_array_explain_string(p3_get_rv_intl(retval)));
 
   if (pa->pick_right_primer == 1 
