@@ -126,6 +126,7 @@ static const double MinEntropyCutoff = -2500.0; /* to filter out non-existing en
 static const double MinEntropy = -3224.0; /* initiation */
 static const double G2 = 0.0; /* structures w higher G are considered to be unstabile */
 const double ABSOLUTE_ZERO = 273.15;
+const double TEMP_KELVIN = 310.15;
 const int MAX_LOOP = 30; /* the maximum size of loop that can be calculated; for larger loops formula must be implemented */
 const int MIN_LOOP = 0;
 static const char BASES[5] = {'A', 'C', 'G', 'T', 'N'}; /* bases to be considered - N is every symbol that is not A, G, C,$
@@ -378,11 +379,11 @@ thal(const unsigned char *oligo_f,
    double* SH;
    int i, j;
    int len_f, len_r;
-   double T1;
    int k;
    int *bp;
    unsigned char *oligo2_rev = NULL;
    double mh, ms;
+   double G1, bestG;
 
    send5 = hend5 = NULL;
    enthalpyDPT = entropyDPT = NULL;
@@ -404,6 +405,8 @@ thal(const unsigned char *oligo_f,
    len_f = length_unsig_char(oligo_f);
    len_r = length_unsig_char(oligo_r);
 
+   /*CHECK_ERROR(1==len_f, "Length 1 first sequence");
+   CHECK_ERROR(1==len_r, "Length 1 second sequence"); */
    /* The following error messages will be seen by end users and will
       not be easy to understand. */
    CHECK_ERROR((len_f > THAL_MAX_ALIGN) && (len_r > THAL_MAX_ALIGN),
@@ -525,7 +528,7 @@ thal(const unsigned char *oligo_f,
 	 /* traceback for unimolecular structure */
 	 drawHairpin(bp, mh, ms, a->temponly,a->temp, o); /* if temponly=1 then return after printing basic therm data */
       } else if(a->temponly==0) {
-	 fputs("No secondary structure could be calculated\n",stderr);
+	    fputs("No secondary structure could be calculated\n",stderr);
       }
 
       if(o->temp==-_INFINITY && (!strcmp(o->msg, ""))) o->temp=0.0;
@@ -545,20 +548,19 @@ thal(const unsigned char *oligo_f,
       entropyDPT = safe_recalloc(entropyDPT, len1, len2, o); /* enthalpyDPT is 3D array represented as 1D array */
       initMatrix();
       fillMatrix(a->maxLoop, o);
-      SHleft = -_INFINITY;
       SH = (double*) safe_malloc(2 * sizeof(double), o);
       /* calculate terminal basepairs */
-      bestI = bestJ = 0;
+      bestI = bestJ = 0; 
+      G1 = bestG = _INFINITY;
       if(a->type==1)
 	for (i = 1; i <= len1; i++) {
 	   for (j = 1; j <= len2; j++) {
 	      RSH(i, j, SH);
 	      SH[0] = SH[0]+SMALL_NON_ZERO; /* this adding is done for compiler, optimization -O2 vs -O0 */
 	      SH[1] = SH[1]+SMALL_NON_ZERO;
-	      T1 = ((EnthalpyDPT(i, j)+ SH[1] + dplx_init_H) / ((EntropyDPT(i, j)) + SH[0] +
-								dplx_init_S + RC)) - ABSOLUTE_ZERO;
-	      if (T1 > SHleft  && ((EntropyDPT(i, j) + SH[0])<0 && (SH[1] + EnthalpyDPT(i, j))<0)) {
-		 SHleft = T1;
+	      G1 = (EnthalpyDPT(i, j)+ SH[1] + dplx_init_H) - TEMP_KELVIN*(EntropyDPT(i, j) + SH[0] + dplx_init_S);  
+	      if(G1<bestG){
+		 bestG = G1;
 		 bestI = i;
 		 bestJ = j;
 	      }
@@ -576,21 +578,20 @@ thal(const unsigned char *oligo_f,
 	 bestI = bestJ = 0;
 	 bestI = len1;
 	 i = len1;
-	 SHleft = -_INFINITY;
+	 G1 = bestG = _INFINITY;
 	 for (j = 1; j <= len2; ++j) {
 	    RSH(i, j, SH);
 	    SH[0] = SH[0]+SMALL_NON_ZERO; /* this adding is done for compiler, optimization -O2 vs -O0,
 					   that compiler could understand that SH is changed in this cycle */
 	    SH[1] = SH[1]+SMALL_NON_ZERO;
-	    T1 = ((EnthalpyDPT(i, j)+ SH[1] + dplx_init_H) / ((EntropyDPT(i, j)) + SH[0] +
-							      dplx_init_S + RC)) - ABSOLUTE_ZERO;
-	    if (T1 > SHleft && ((SH[0] + EntropyDPT(i, j))<0 && (SH[1] + EnthalpyDPT(i, j))<0)) {
-	       SHleft = T1;
-	       bestJ = j;
+	    G1 = (EnthalpyDPT(i, j)+ SH[1] + dplx_init_H) - TEMP_KELVIN*(EntropyDPT(i, j) + SH[0] + dplx_init_S);  
+		if(G1<bestG){
+		   bestG = G1;
+      	           bestJ = j;
 	    }
 	 }
       }
-      if (!isFinite(SHleft)) bestI = bestJ = 1;
+      if (!isFinite(bestG)) bestI = bestJ = 1;
       double dH, dS;
       RSH(bestI, bestJ, SH);
       dH = EnthalpyDPT(bestI, bestJ)+ SH[1] + dplx_init_H;
@@ -607,6 +608,7 @@ thal(const unsigned char *oligo_f,
 	 o->align_end_2=bestJ;
       } else  {
 	 o->temp = 0.0;
+	 /* fputs("No secondary structure could be calculated\n",stderr); */
       }
       free(ps1);
       free(ps2);
@@ -635,7 +637,7 @@ set_thal_default_args(thal_args *a)
    a->dv = 0.0; /* mM */
    a->dntp = 0.8; /* mM */
    a->dna_conc = 50; /* nM */
-   a->temp = 310.15; /* Kelvin */
+   a->temp = TEMP_KELVIN; /* Kelvin */
    a->temponly = 1; /* return only melting temperature of predicted structure */
    a->dimer = 1; /* by default dimer structure is calculated */
 }
@@ -652,7 +654,7 @@ set_thal_oligo_default_args(thal_args *a)
    a->dv = 0.0; /* mM */
    a->dntp = 0.0; /* mM */
    a->dna_conc = 50; /* nM */
-   a->temp = 310.15; /* Kelvin */
+   a->temp = TEMP_KELVIN; /* Kelvin */
    a->temponly = 1; /* return only melting temperature of predicted structure */
    a->dimer = 1; /* by default dimer structure is calculated */
 }
@@ -1367,18 +1369,23 @@ maxTM(int i, int j)
    double T0, T1;
    double S0, S1;
    double H0, H1;
+   double* SH;
+   SH = (double*) safe_malloc(2 * sizeof(double), 0);
    T0 = T1 = -_INFINITY;
    S0 = EntropyDPT(i, j);
    H0 = EnthalpyDPT(i, j);
-   T0 = (H0 + dplx_init_H) /(S0 + dplx_init_S + RC); /* at current position */
+   RSH(i,j,SH);
+   T0 = (H0 + dplx_init_H + SH[1]) /(S0 + dplx_init_S + SH[0] + RC); /* at current position */
    if(isFinite(EnthalpyDPT(i - 1, j - 1)) && isFinite(Hs(i - 1, j - 1, 1))) {
       S1 = (EntropyDPT(i - 1, j - 1) + Ss(i - 1, j - 1, 1));
       H1 = (EnthalpyDPT(i - 1, j - 1) + Hs(i - 1, j - 1, 1));
+      T1 = (H1 + dplx_init_H + SH[1]) /(S1 + dplx_init_S + SH[0] + RC);
    } else {
       S1 = -1.0;
       H1 = _INFINITY;
+      T1 = (H1 + dplx_init_H) /(S1 + dplx_init_S + RC);
    }
-   T1 = (H1 + dplx_init_H) /(S1 + dplx_init_S + RC);
+   
    if(S1 < MinEntropyCutoff) {
       /* to not give dH any value if dS is unreasonable */
       S1 = MinEntropy;
@@ -1389,13 +1396,14 @@ maxTM(int i, int j)
       S0 = MinEntropy;
       H0 = 0.0;
    }
-   if((T1 > T0) || (S0>0 && H0>0)){ /* T1 on suurem */
+   if(T1 > T0) { 
       EntropyDPT(i, j) = S1;
       EnthalpyDPT(i, j) = H1;
    } else if(T0 >= T1) {
       EntropyDPT(i, j) = S0;
       EnthalpyDPT(i, j) = H0;
    }
+   free(SH);
 }
 
 static void 
@@ -1438,8 +1446,8 @@ maxTM2(int i, int j)
 static void 
 LSH(int i, int j, double* EntropyEnthalpy)
 {
-   double S1, H1, T1;
-   double S2, H2, T2;
+   double S1, H1, T1, G1;
+   double S2, H2, T2, G2;
    S1 = S2 = -1.0;
    H1 = H2 = -_INFINITY;
    T1 = T2 = -_INFINITY;
@@ -1450,69 +1458,77 @@ LSH(int i, int j, double* EntropyEnthalpy)
    }
    S1 = atPenaltyS(numSeq1[i], numSeq2[j]) + tstack2Entropies[numSeq2[j]][numSeq2[j-1]][numSeq1[i]][numSeq1[i-1]];
    H1 = atPenaltyH(numSeq1[i], numSeq2[j]) + tstack2Enthalpies[numSeq2[j]][numSeq2[j-1]][numSeq1[i]][numSeq1[i-1]];
-   if(!isFinite(H1)) {
+   G1 = H1 - TEMP_KELVIN*S1;
+   if(!isFinite(H1) || G1>0) {
       H1 = _INFINITY;
       S1 = -1.0;
+      G1 = 1.0;
    }
    /** If there is two dangling ends at the same end of duplex **/
-   if(isFinite(dangleEnthalpies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]]) && isFinite(dangleEnthalpies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]])) {
+   if((bpIndx(numSeq1[i-1], numSeq2[j-1]) != 1 ) && isFinite(dangleEnthalpies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]]) && isFinite(dangleEnthalpies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]])) {
       S2 = atPenaltyS(numSeq1[i], numSeq2[j]) + dangleEntropies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]] +
 	dangleEntropies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]];
       H2 = atPenaltyH(numSeq1[i], numSeq2[j]) + dangleEnthalpies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]] +
 	dangleEnthalpies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]];
-      if(!isFinite(H2)) {
+      G2 = H2 - TEMP_KELVIN*S2;
+      if(!isFinite(H2) || G2>0) {
 	 H2 = _INFINITY;
 	 S2 = -1.0;
+     G2 = 1.0;
       }
       T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
-      if(isFinite(H1)) {
+      if(isFinite(H1) && G1<0) {
 	 T1 = (H1 + dplx_init_H) / (S1 + dplx_init_S + RC);
-	 if(T1<T2) {
+	 if(T1<T2 && G2<0) {
 	    S1 = S2;
 	    H1 = H2;
 	    T1 = T2;
 	 }
-      } else {
+      } else if(G2<0) {
 	 S1 = S2;
 	 H1 = H2;
 	 T1 = T2;
       }
-   } else if (isFinite(dangleEnthalpies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]])) {
+   } else if ((bpIndx(numSeq1[i-1], numSeq2[j-1]) != 1) && isFinite(dangleEnthalpies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]])) {
       S2 = atPenaltyS(numSeq1[i], numSeq2[j]) + dangleEntropies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]];
       H2 = atPenaltyH(numSeq1[i], numSeq2[j]) + dangleEnthalpies3[numSeq2[j]][numSeq2[j - 1]][numSeq1[i]];
-      if(!isFinite(H2)) {
+      G2 = H2 - TEMP_KELVIN*S2;
+      if(!isFinite(H2) || G2>0) {
 	 H2 = _INFINITY;
 	 S2 = -1.0;
+     G2 = 1.0;
       }
       T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
-      if(isFinite(H1)) {
+      if(isFinite(H1) && G1<0) {
 	 T1 = (H1 + dplx_init_H) / (S1 + dplx_init_S + RC);
-	 if(T1<T2) {
+	 if(T1<T2 && G2<0) {
 	    S1 = S2;
 	    H1 = H2;
 	    T1 = T2;
 	 }
-      } else {
+      } else if (G2<0){
 	 S1 = S2;
 	 H1 = H2;
 	 T1 = T2;
       }
-   } else if (isFinite(dangleEnthalpies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]])) {
+   } else if ((bpIndx(numSeq1[i-1], numSeq2[j-1]) != 1) && isFinite(dangleEnthalpies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]])) {
       S2 = atPenaltyS(numSeq1[i], numSeq2[j]) + dangleEntropies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]];
       H2 = atPenaltyH(numSeq1[i], numSeq2[j]) + dangleEnthalpies5[numSeq2[j]][numSeq1[i]][numSeq1[i - 1]];
-      if(!isFinite(H2)) {
+      G2 = H2 - TEMP_KELVIN*S2;
+      if(!isFinite(H2) || G2>0) {
 	 H2 = _INFINITY;
 	 S2 = -1.0;
+     G2 = 1.0;     
       }
       T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
-      if(isFinite(H1)) {
+      if(isFinite(H1) && G1<0) {
 	 T1 = (H1 + dplx_init_H) / (S1 + dplx_init_S + RC);
-	 if(T1 < T2) {
+	 if(T1 < T2  && G2<0) {
 	    S1 = S2;
 	    H1 = H2;
 	    T1 = T2;
 	 }
-      } else {
+      } else if(G2<0) {
 	 S1 = S2;
 	 H1 = H2;
 	 T1 = T2;
@@ -1521,6 +1537,8 @@ LSH(int i, int j, double* EntropyEnthalpy)
    S2 = atPenaltyS(numSeq1[i], numSeq2[j]);
    H2 = atPenaltyH(numSeq1[i], numSeq2[j]);
    T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
+   G1 = H1 -TEMP_KELVIN*S1;   
+   G2 = H2 -TEMP_KELVIN*S2;
    if(isFinite(H1)) {
       if(T1 < T2) {
 	 EntropyEnthalpy[0] = S2;
@@ -1539,6 +1557,7 @@ LSH(int i, int j, double* EntropyEnthalpy)
 static void 
 RSH(int i, int j, double* EntropyEnthalpy)
 {
+   double G1, G2;
    double S1, S2;
    double H1, H2;
    double T1, T2;
@@ -1552,74 +1571,82 @@ RSH(int i, int j, double* EntropyEnthalpy)
    }
    S1 = atPenaltyS(numSeq1[i], numSeq2[j]) + tstack2Entropies[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]][numSeq2[j + 1]];
    H1 = atPenaltyH(numSeq1[i], numSeq2[j]) + tstack2Enthalpies[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]][numSeq2[j + 1]];
-   if(!isFinite(H1)) {
+   G1 = H1 - TEMP_KELVIN*S1;
+   if(!isFinite(H1) || G1>0) {
       H1 = _INFINITY;
       S1 = -1.0;
+      G1 = 1.0;
    }
    
-   if(isFinite(dangleEnthalpies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]]) && isFinite(dangleEnthalpies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]])) {
+   if(bpIndx(numSeq1[i+1], numSeq2[j+1]) == 0 && isFinite(dangleEnthalpies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]]) && isFinite(dangleEnthalpies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]])) {
       S2 = atPenaltyS(numSeq1[i], numSeq2[j]) + dangleEntropies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]] +
 	dangleEntropies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]];
       H2 = atPenaltyH(numSeq1[i], numSeq2[j]) + dangleEnthalpies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]] +
 	dangleEnthalpies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]];
-      if(!isFinite(H2)) {
+    G2 = H2 - TEMP_KELVIN*S2;
+      if(!isFinite(H2) || G2>0) {
 	 H2 = _INFINITY;
 	 S2 = -1.0;
+     G2 = 1.0;
       }
       
       T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
-      if(isFinite(H1)) {
+      if(isFinite(H1) && G1<0) {
 	 T1 = (H1 + dplx_init_H) / (S1 + dplx_init_S + RC);
-	 if(T1 < T2) {
+	 if(T1 < T2 && G2<0) {
 	    S1 = S2;
 	    H1 = H2;
 	    T1 = T2;
 	 }
-      } else {
+      } else if(G2<0){
 	 S1 = S2;
 	 H1 = H2;
 	 T1 = T2;
       }
    }
 
-   if(isFinite(dangleEnthalpies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]])) {
+   else if(bpIndx(numSeq1[i+1], numSeq2[j+1]) == 0 && isFinite(dangleEnthalpies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]])) {
       S2 = atPenaltyS(numSeq1[i], numSeq2[j]) + dangleEntropies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]];
       H2 = atPenaltyH(numSeq1[i], numSeq2[j]) + dangleEnthalpies3[numSeq1[i]][numSeq1[i + 1]][numSeq2[j]];
-      if(!isFinite(H2)) {
+      G2 = H2 - TEMP_KELVIN*S2;
+      if(!isFinite(H2) || G2 >0) {
 	 H2 = _INFINITY;
 	 S2 = -1.0;
+     G2 = 1.0;
       }
       T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
-      if(isFinite(H1)) {
+      if(isFinite(H1) && G1<0) {
 	 T1 = (H1 + dplx_init_H) / (S1 + dplx_init_S + RC);
-	 if(T1 < T2) {
+	 if(T1 < T2 && G2<0) {
 	    S1 = S2;
 	    H1 = H2;
 	    T1 = T2;
 	 }
-      } else {
+      } else if(G2<0){
 	 S1 = S2;
 	 H1 = H2;
 	 T1 = T2;
       }
    }
 
-   if(isFinite(dangleEnthalpies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]])) {
+   else if(bpIndx(numSeq1[i+1], numSeq2[j+1]) == 0 && isFinite(dangleEnthalpies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]])) {
       S2 = atPenaltyS(numSeq1[i], numSeq2[j]) + dangleEntropies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]];
       H2 = atPenaltyH(numSeq1[i], numSeq2[j]) + dangleEnthalpies5[numSeq1[i]][numSeq2[j]][numSeq2[j + 1]];
-      if(!isFinite(H2)) {
+      G2 = H2 - TEMP_KELVIN*S2;
+      if(!isFinite(H2) || G2>0) {
 	 H2 = _INFINITY;
 	 S2 = -1.0;
+     G2 = 1.0;
       }
       T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
-      if(isFinite(H1)) {
+      if(isFinite(H1) && G1<0) {
 	 T1 = (H1 + dplx_init_H) / (S1 + dplx_init_S + RC);
-	 if(T1 < T2) {
+	 if(T1 < T2 && G2<0) {
 	    S1 = S2;
 	    H1 = H2;
 	    T1 = T2;
 	 }
-      } else {
+      } else if (G2<0){
 	 S1 = S2;
 	 H1 = H2;
 	 T1 = T2;
@@ -1628,6 +1655,8 @@ RSH(int i, int j, double* EntropyEnthalpy)
    S2 = atPenaltyS(numSeq1[i], numSeq2[j]);
    H2 = atPenaltyH(numSeq1[i], numSeq2[j]);
    T2 = (H2 + dplx_init_H) / (S2 + dplx_init_S + RC);
+   G1 = H1 -TEMP_KELVIN*S1;
+   G2 =  H2 -TEMP_KELVIN*S2;
    if(isFinite(H1)) {
       if(T1 < T2) {
 	 EntropyEnthalpy[0] = S2;
@@ -1697,7 +1726,7 @@ CBI(int i, int j, double* EntropyEnthalpy, int traceback, int maxLoop)
 	   EntropyEnthalpy[0] = -1.0;
 	   EntropyEnthalpy[1] = _INFINITY;
 	}
-	if (isFinite(EnthalpyDPT(ii, jj))) {
+	if (isFinite(EnthalpyDPT(ii, jj)) && isFinite(EnthalpyDPT(i, j))) {
 	   calc_bulge_internal2(i, j, ii, jj, EntropyEnthalpy, traceback,maxLoop);
 	   if(isFinite(EntropyEnthalpy[1])) {
 	      if(EntropyEnthalpy[0] < MinEntropyCutoff) {
@@ -1718,8 +1747,12 @@ static void
 calc_hairpin(int i, int j, double* EntropyEnthalpy, int traceback)
 {
    int loopSize = j - i - 1;
-   double T1, T2;
-   T1 = T2 = -_INFINITY;
+   double G1, G2;
+   G1 = G2 = -_INFINITY;
+   double* SH;
+   SH = (double*) safe_malloc(2 * sizeof(double), 0);
+   SH[0] = -1.0;
+   SH[1] = _INFINITY;
    if(loopSize < MIN_HRPN_LOOP) {
       EntropyEnthalpy[0] = -1.0;
       EntropyEnthalpy[1] = _INFINITY;
@@ -1776,12 +1809,14 @@ calc_hairpin(int i, int j, double* EntropyEnthalpy, int traceback)
       EntropyEnthalpy[1] = _INFINITY;
       EntropyEnthalpy[0] = -1.0;
    }
-   T1 = (EntropyEnthalpy[1] + dplx_init_H) / ((EntropyEnthalpy[0] + dplx_init_S + RC));
-   T2 = (EnthalpyDPT(i, j) + dplx_init_H) / ((EntropyDPT(i, j)) + dplx_init_S + RC);
-   if(T1 < T2 && traceback == 0) {
+   RSH(i,j,SH);
+   G1 = EntropyEnthalpy[1]+SH[1] -TEMP_KELVIN*(EntropyEnthalpy[0]+SH[0]);
+   G2 = EnthalpyDPT(i, j)+SH[1] -TEMP_KELVIN*(EntropyDPT(i, j)+SH[0]);
+     if(G2 < G1 && traceback == 0) {
       EntropyEnthalpy[0] = EntropyDPT(i, j);
       EntropyEnthalpy[1] = EnthalpyDPT(i, j);
    }
+   free(SH);
    return;
 }
 
@@ -1790,12 +1825,14 @@ static void
 calc_bulge_internal(int i, int j, int ii, int jj, double* EntropyEnthalpy, int traceback, int maxLoop)
 {
    int loopSize1, loopSize2, loopSize;
-   double T1, T2;
-   double S,H;
+   double S,H,G1,G2;
    int N, N_loop;
-   T1 = T2 = -_INFINITY;
-   S = MinEntropy;
-   H = 0;
+   double* SH;
+   SH = (double*) safe_malloc(2 * sizeof(double), 0);
+   SH[0] = -1.0;
+   SH[1] = _INFINITY;
+   S = -1.0;
+   H = _INFINITY;
    loopSize1 = ii - i - 1;
    loopSize2 = jj - j - 1;
    if(ii < jj) {
@@ -1821,16 +1858,18 @@ calc_bulge_internal(int i, int j, int ii, int jj, double* EntropyEnthalpy, int t
 #ifdef DEBUG
    if (loopSize1 + loopSize2 > maxLoop) {
       fputs("Error: calc_bulge_internal() called with loopSize1 + loopSize2 > maxLoop\n", stderr);
+      free(SH);
       return;
    }
 #endif
 #ifdef DEBUG
    if (loopSize1 == 0 && loopSize2 == 0) {
       fputs("Error: calc_bulge_internal() called with nonsense\n", stderr);
+      free(SH);
       return;
    }
 #endif
-   loopSize = loopSize1 + loopSize2 -1;
+   loopSize = loopSize1 + loopSize2-1;
    if((loopSize1 == 0 && loopSize2 > 0) || (loopSize2 == 0 && loopSize1 > 0)) { /* only bulges have to be considered */
       if(loopSize2 == 1 || loopSize1 == 1) { /* bulge loop of size one is treated differently
 					      the intervening nn-pair must be added */
@@ -1841,20 +1880,20 @@ calc_bulge_internal(int i, int j, int ii, int jj, double* EntropyEnthalpy, int t
 	    S = bulgeLoopEntropies[loopSize] +
 	      stackEntropies[numSeq1[i]][numSeq1[ii]][numSeq2[j]][numSeq2[jj]];
 	 }
+	 if(isPositive(H) || isPositive(S)){
+	    H = _INFINITY;
+	    S = -1.0;
+	 }
 	 H += EnthalpyDPT(i, j);
 	 S += EntropyDPT(i, j);
 	 if(!isFinite(H)) {
 	    H = _INFINITY;
 	    S = -1.0;
 	 }
-	 if(isPositive(H) && isPositive(S) && (!isPositive(EnthalpyDPT(ii, jj)) || !isPositive(EntropyDPT(ii, jj)))) { /* if both, S and H are positive */
-	    H = _INFINITY;
-	    S = -1.0;
-	 }
-	 T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
-	 T2 = (EnthalpyDPT(ii, jj) + dplx_init_H) / ((EntropyDPT(ii, jj)) + dplx_init_S + RC);
-
-	 if((T1 > T2) || ((traceback && T1 >= T2) || (traceback==1))) {
+	 RSH(ii,jj,SH);
+	 G1 = H+SH[1] -TEMP_KELVIN*(S+SH[0]);
+	 G2 = EnthalpyDPT(ii, jj)+SH[1]-TEMP_KELVIN*((EntropyDPT(ii, jj)+SH[0]));
+	 if((G1< G2) || (traceback==1)) {
 	    EntropyEnthalpy[0] = S;
 	    EntropyEnthalpy[1] = H;
 	 }
@@ -1869,16 +1908,19 @@ calc_bulge_internal(int i, int j, int ii, int jj, double* EntropyEnthalpy, int t
 	    H = _INFINITY;
 	    S = -1.0;
 	 }
-	 if(isPositive(H) && isPositive(S) && (!isPositive(EnthalpyDPT(i, j)) || !isPositive(EntropyDPT(i, j)))){ /* if both, S and H are positive */
+	 if(isPositive(H) && isPositive(S)){ 
 	    H = _INFINITY;
 	    S = -1.0;
 	 }
-	 T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
-	 T2 = (EnthalpyDPT(ii, jj) + dplx_init_H) / (EntropyDPT(ii, jj) + dplx_init_S + RC);
-	 if((T1 > T2) || ((traceback && T1 >= T2) || (traceback==1))) {
+	
+     RSH(ii,jj,SH);
+	 G1 = H+SH[1] -TEMP_KELVIN*(S+SH[0]);
+	 G2 = EnthalpyDPT(ii, jj)+SH[1]-TEMP_KELVIN*(EntropyDPT(ii, jj)+SH[0]);
+	 if(G1< G2 || (traceback==1)){
 	    EntropyEnthalpy[0] = S;
 	    EntropyEnthalpy[1] = H;
 	 }
+	 
       }
    } else if (loopSize1 == 1 && loopSize2 == 1) {
       S = stackint2Entropies[numSeq1[i]][numSeq1[i+1]][numSeq2[j]][numSeq2[j+1]] +
@@ -1892,19 +1934,18 @@ calc_bulge_internal(int i, int j, int ii, int jj, double* EntropyEnthalpy, int t
 	 H = _INFINITY;
 	 S = -1.0;
       }
-      if(isPositive(H) && isPositive(S) && (!isPositive(EnthalpyDPT(ii, jj)) || !isPositive(EntropyDPT(ii, jj)))) { /* if both, S and H are positive */
+      if(isPositive(H) && isPositive(S)){
 	 H = _INFINITY;
 	 S = -1.0;
       }    
-      T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
-      T2 = (EnthalpyDPT(ii, jj) + dplx_init_H) / (EntropyDPT(ii, jj) + dplx_init_S + RC);
-
-      if((DBL_EQ(T1,T2) == 2) || traceback==1) {
-	 if((T1 > T2) || (traceback && T1 >= T2)) {
+     RSH(ii,jj,SH);
+	 G1 = H+SH[1] -TEMP_KELVIN*(S+SH[0]);
+      G2 = EnthalpyDPT(ii, jj)+SH[1]-TEMP_KELVIN*(EntropyDPT(ii, jj)+SH[0]);
+	   if((G1< G2) || traceback==1) {
 	    EntropyEnthalpy[0] = S;
 	    EntropyEnthalpy[1] = H;
 	 }
-      }
+      free(SH);
       return;
    } else { /* only internal loops */
       H = interiorLoopEnthalpies[loopSize] + tstackEnthalpies[numSeq1[i]][numSeq1[i+1]][numSeq2[j]][numSeq2[j+1]] +
@@ -1919,17 +1960,19 @@ calc_bulge_internal(int i, int j, int ii, int jj, double* EntropyEnthalpy, int t
 	 H = _INFINITY;
 	 S = -1.0;
       }
-      if(isPositive(H) && isPositive(S) && (!isPositive(EnthalpyDPT(ii, jj)) || !isPositive(EntropyDPT(ii, jj)))) { /* if both, S and H are positive */
+   if(isPositive(H) && isPositive(S)){ 
 	 H = _INFINITY;
 	 S = -1.0;
       }
-      T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
-      T2 = (EnthalpyDPT(ii, jj) + dplx_init_H) / ((EntropyDPT(ii, jj)) + dplx_init_S + RC);
-      if((T1 > T2) || ((traceback && T1 >= T2) || (traceback==1))) {
-	 EntropyEnthalpy[0] = S;
-	 EntropyEnthalpy[1] = H;
+     RSH(ii,jj,SH);
+     G1 = H+SH[1] -TEMP_KELVIN*(S+SH[0]);
+     G2 = EnthalpyDPT(ii, jj)+SH[1]-TEMP_KELVIN*(EntropyDPT(ii, jj)+SH[0]);
+     if((G1< G2) || (traceback==1)){
+	     EntropyEnthalpy[0] = S;
+	     EntropyEnthalpy[1] = H;
       }
    }
+   free(SH);
    return;
 }
 
@@ -1951,13 +1994,13 @@ calc_bulge_internal2(int i, int j, int ii, int jj, double* EntropyEnthalpy, int
       return;
    }
    /* Triinu, please review the statements below. */
-   if(i < (len1 -j)) {
+   /* if(i < (len1 -j)) { */
      /* N  = i; */
       /* N_loop = (i - 1); */
-   } else {
+   /* } else { */
      /* N = len1-j;  */
       /* N_loop = len1 - j - 1; */
-   }
+   /* } */
 #ifdef DEBUG
    if (ii <= i)
      fputs("Error in calc_bulge_internal(): ii isn't greater than i\n", stderr);
@@ -2015,10 +2058,7 @@ calc_bulge_internal2(int i, int j, int ii, int jj, double* EntropyEnthalpy, int
 	    H = _INFINITY;
 	    S = -1.0;
 	 }
-	 if(isPositive(H) && isPositive(S) && (!isPositive(EnthalpyDPT(i, j)) || !isPositive(EntropyDPT(i, j)))) { /* if both, S and H are positive */
-	    H = _INFINITY;
-	    S = -1.0;
-	 }
+	
 	 T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
 	 T2 = (EnthalpyDPT(i, j) + dplx_init_H) / ((EntropyDPT(i, j)) + dplx_init_S + RC);
 
@@ -2040,10 +2080,7 @@ calc_bulge_internal2(int i, int j, int ii, int jj, double* EntropyEnthalpy, int
 	    H = _INFINITY;
 	    S = -1.0;
 	 }
-	 if(isPositive(H) && isPositive(S) && (!isPositive(EnthalpyDPT(i, j)) || !isPositive(EntropyDPT(i, j)))) { /* if both, S and H are positive */
-	    H = _INFINITY;
-	    S = -1.0;
-	 }	 
+	  
 	 T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
 	 T2 = (EnthalpyDPT(i, j) + dplx_init_H) / (EntropyDPT(i, j) + dplx_init_S + RC);
 
@@ -2069,10 +2106,7 @@ calc_bulge_internal2(int i, int j, int ii, int jj, double* EntropyEnthalpy, int
 	 H = _INFINITY;
 	 S = -1.0;
       }
-      if(isPositive(H)==1 && isPositive(S)==1 && (!isPositive(EnthalpyDPT(i, j)) || !isPositive(EntropyDPT(i, j)))) { /* if both, S and H are positive */
-	 H = _INFINITY;
-	 S = -1.0;
-      }
+    
       T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
       T2 = (EnthalpyDPT(i, j) + dplx_init_H) / (EntropyDPT(i, j) + dplx_init_S + RC);
       if((DBL_EQ(T1,T2) == 2) || traceback) {
@@ -2098,10 +2132,7 @@ calc_bulge_internal2(int i, int j, int ii, int jj, double* EntropyEnthalpy, int
 	 H = _INFINITY;
 	 S = -1.0;
       }
-      if(isPositive(H) && isPositive(S) && (!isPositive(EnthalpyDPT(i, j)) || !isPositive(EntropyDPT(i, j)))){ /* if both, S and H are positive */
-	 H = _INFINITY;
-	 S = -1.0;
-      }
+    
       T1 = (H + dplx_init_H) / ((S + dplx_init_S) + RC);
       T2 = (EnthalpyDPT(i, j) + dplx_init_H) / ((EntropyDPT(i, j)) + dplx_init_S + RC);
       if((T1 > T2) || ((traceback && T1 >= T2) || (traceback==1))) {
